@@ -6,12 +6,14 @@ use vars qw($VERSION);
 $VERSION = '0.02';
 
 sub new {
+    use attrs 'method';
     # a unique $id could be constructed with join('.', `hostname`, $$, $id++);
 
     my ($class, $near, $id, $priority) = @_;
     # uses real pointers so must be per-database...
     my $t = $near->database_of->hash->{'ObjStore::Job::Table'};
-    my $o = shift->SUPER::new($t);
+    die "No ObjStore::Job::Table found" if !$t;
+    my $o = shift->SUPER::new($t->segment_of);
     # $id is a string!
     if ($id) { $$o{id} = "$id"; }
     else {     $$o{id} = "$$t{nextid}"; ++$$t{nextid}; }
@@ -25,10 +27,12 @@ sub new {
 }
 
 sub runnable {
+    use attrs 'method';
     my $state = shift->{state};
     $state ne 'D' and $state ne 'K';
 }
 sub running {
+    use attrs 'method';
     my $state = shift->{state};
     $state eq 'R' or $state eq 'S';
 }
@@ -68,6 +72,7 @@ sub do_acknowledge {  #like wait(2)
 }
 
 sub cancel {
+    use attrs 'method';
     my ($o) = @_;
     $$o{job_table}->focus->remove($o);
 }
@@ -121,7 +126,7 @@ Allowed to consume all available pizza slices.
 =item * TIME-SLICED 1-20
 
 Given pizza slices proportional to the priority until either all the
-pizza slices are consumed or all the jobs are asleep (feast induced
+pizza slices are consumed or all the jobs are asleep (feasts induce
 slumber :-).
 
 =item * IDLE > 20
@@ -136,7 +141,7 @@ The whole scheduling operation occurs within a single transaction.
 While this means that any job can kill the entire transaction, this
 seems a better choice than wrapping every job in its own
 mini-transaction.  Since transactions are relatively expensive, it is
-assumed that most of the time all jobs will complete without error.
+hoped that most of the time all jobs will complete without error.
 
 =head1 BUGS
 
